@@ -1,18 +1,32 @@
 # dialects/mysql/types.py
-# Copyright (C) 2005-2024 the SQLAlchemy authors and contributors
+# Copyright (C) 2005-2026 the SQLAlchemy authors and contributors
 # <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: https://www.opensource.org/licenses/mit-license.php
+from __future__ import annotations
 
 import datetime
+import decimal
+from typing import Any
+from typing import Iterable
+from typing import Optional
+from typing import TYPE_CHECKING
+from typing import Union
 
 from ... import exc
-from ... import types as sqltypes
 from ... import util
+from ...sql import sqltypes
+
+if TYPE_CHECKING:
+    from .base import MySQLDialect
+    from ...engine.interfaces import Dialect
+    from ...sql.type_api import _BindProcessorType
+    from ...sql.type_api import _ResultProcessorType
+    from ...sql.type_api import TypeEngine
 
 
-class _NumericType(object):
+class _NumericType:
     """Base for MySQL numeric types.
 
     This is the base both for NUMERIC as well as INTEGER, hence
@@ -20,19 +34,27 @@ class _NumericType(object):
 
     """
 
-    def __init__(self, unsigned=False, zerofill=False, **kw):
+    def __init__(
+        self, unsigned: bool = False, zerofill: bool = False, **kw: Any
+    ):
         self.unsigned = unsigned
         self.zerofill = zerofill
-        super(_NumericType, self).__init__(**kw)
+        super().__init__(**kw)
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return util.generic_repr(
             self, to_inspect=[_NumericType, sqltypes.Numeric]
         )
 
 
-class _FloatType(_NumericType, sqltypes.Float):
-    def __init__(self, precision=None, scale=None, asdecimal=True, **kw):
+class _FloatType(_NumericType, sqltypes.Float[Union[decimal.Decimal, float]]):
+    def __init__(
+        self,
+        precision: Optional[int] = None,
+        scale: Optional[int] = None,
+        asdecimal: bool = True,
+        **kw: Any,
+    ):
         if isinstance(self, (REAL, DOUBLE)) and (
             (precision is None and scale is not None)
             or (precision is not None and scale is None)
@@ -41,23 +63,21 @@ class _FloatType(_NumericType, sqltypes.Float):
                 "You must specify both precision and scale or omit "
                 "both altogether."
             )
-        super(_FloatType, self).__init__(
-            precision=precision, asdecimal=asdecimal, **kw
-        )
+        super().__init__(precision=precision, asdecimal=asdecimal, **kw)
         self.scale = scale
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return util.generic_repr(
             self, to_inspect=[_FloatType, _NumericType, sqltypes.Float]
         )
 
 
 class _IntegerType(_NumericType, sqltypes.Integer):
-    def __init__(self, display_width=None, **kw):
+    def __init__(self, display_width: Optional[int] = None, **kw: Any):
         self.display_width = display_width
-        super(_IntegerType, self).__init__(**kw)
+        super().__init__(**kw)
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return util.generic_repr(
             self, to_inspect=[_IntegerType, _NumericType, sqltypes.Integer]
         )
@@ -68,13 +88,13 @@ class _StringType(sqltypes.String):
 
     def __init__(
         self,
-        charset=None,
-        collation=None,
-        ascii=False,  # noqa
-        binary=False,
-        unicode=False,
-        national=False,
-        **kw
+        charset: Optional[str] = None,
+        collation: Optional[str] = None,
+        ascii: bool = False,  # noqa
+        binary: bool = False,
+        unicode: bool = False,
+        national: bool = False,
+        **kw: Any,
     ):
         self.charset = charset
 
@@ -85,27 +105,35 @@ class _StringType(sqltypes.String):
         self.unicode = unicode
         self.binary = binary
         self.national = national
-        super(_StringType, self).__init__(**kw)
+        super().__init__(**kw)
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return util.generic_repr(
             self, to_inspect=[_StringType, sqltypes.String]
         )
 
 
-class _MatchType(sqltypes.Float, sqltypes.MatchType):
-    def __init__(self, **kw):
+class _MatchType(
+    sqltypes.Float[Union[decimal.Decimal, float]], sqltypes.MatchType
+):
+    def __init__(self, **kw: Any):
         # TODO: float arguments?
-        sqltypes.Float.__init__(self)
+        sqltypes.Float.__init__(self)  # type: ignore[arg-type]
         sqltypes.MatchType.__init__(self)
 
 
-class NUMERIC(_NumericType, sqltypes.NUMERIC):
+class NUMERIC(_NumericType, sqltypes.NUMERIC[Union[decimal.Decimal, float]]):
     """MySQL NUMERIC type."""
 
     __visit_name__ = "NUMERIC"
 
-    def __init__(self, precision=None, scale=None, asdecimal=True, **kw):
+    def __init__(
+        self,
+        precision: Optional[int] = None,
+        scale: Optional[int] = None,
+        asdecimal: bool = True,
+        **kw: Any,
+    ):
         """Construct a NUMERIC.
 
         :param precision: Total digits in this number.  If scale and precision
@@ -121,17 +149,23 @@ class NUMERIC(_NumericType, sqltypes.NUMERIC):
           numeric.
 
         """
-        super(NUMERIC, self).__init__(
+        super().__init__(
             precision=precision, scale=scale, asdecimal=asdecimal, **kw
         )
 
 
-class DECIMAL(_NumericType, sqltypes.DECIMAL):
+class DECIMAL(_NumericType, sqltypes.DECIMAL[Union[decimal.Decimal, float]]):
     """MySQL DECIMAL type."""
 
     __visit_name__ = "DECIMAL"
 
-    def __init__(self, precision=None, scale=None, asdecimal=True, **kw):
+    def __init__(
+        self,
+        precision: Optional[int] = None,
+        scale: Optional[int] = None,
+        asdecimal: bool = True,
+        **kw: Any,
+    ):
         """Construct a DECIMAL.
 
         :param precision: Total digits in this number.  If scale and precision
@@ -147,17 +181,23 @@ class DECIMAL(_NumericType, sqltypes.DECIMAL):
           numeric.
 
         """
-        super(DECIMAL, self).__init__(
+        super().__init__(
             precision=precision, scale=scale, asdecimal=asdecimal, **kw
         )
 
 
-class DOUBLE(_FloatType):
+class DOUBLE(_FloatType, sqltypes.DOUBLE[Union[decimal.Decimal, float]]):
     """MySQL DOUBLE type."""
 
     __visit_name__ = "DOUBLE"
 
-    def __init__(self, precision=None, scale=None, asdecimal=True, **kw):
+    def __init__(
+        self,
+        precision: Optional[int] = None,
+        scale: Optional[int] = None,
+        asdecimal: bool = True,
+        **kw: Any,
+    ):
         """Construct a DOUBLE.
 
         .. note::
@@ -181,17 +221,23 @@ class DOUBLE(_FloatType):
           numeric.
 
         """
-        super(DOUBLE, self).__init__(
+        super().__init__(
             precision=precision, scale=scale, asdecimal=asdecimal, **kw
         )
 
 
-class REAL(_FloatType, sqltypes.REAL):
+class REAL(_FloatType, sqltypes.REAL[Union[decimal.Decimal, float]]):
     """MySQL REAL type."""
 
     __visit_name__ = "REAL"
 
-    def __init__(self, precision=None, scale=None, asdecimal=True, **kw):
+    def __init__(
+        self,
+        precision: Optional[int] = None,
+        scale: Optional[int] = None,
+        asdecimal: bool = True,
+        **kw: Any,
+    ):
         """Construct a REAL.
 
         .. note::
@@ -215,17 +261,23 @@ class REAL(_FloatType, sqltypes.REAL):
           numeric.
 
         """
-        super(REAL, self).__init__(
+        super().__init__(
             precision=precision, scale=scale, asdecimal=asdecimal, **kw
         )
 
 
-class FLOAT(_FloatType, sqltypes.FLOAT):
+class FLOAT(_FloatType, sqltypes.FLOAT[Union[decimal.Decimal, float]]):
     """MySQL FLOAT type."""
 
     __visit_name__ = "FLOAT"
 
-    def __init__(self, precision=None, scale=None, asdecimal=False, **kw):
+    def __init__(
+        self,
+        precision: Optional[int] = None,
+        scale: Optional[int] = None,
+        asdecimal: bool = False,
+        **kw: Any,
+    ):
         """Construct a FLOAT.
 
         :param precision: Total digits in this number.  If scale and precision
@@ -241,11 +293,13 @@ class FLOAT(_FloatType, sqltypes.FLOAT):
           numeric.
 
         """
-        super(FLOAT, self).__init__(
+        super().__init__(
             precision=precision, scale=scale, asdecimal=asdecimal, **kw
         )
 
-    def bind_processor(self, dialect):
+    def bind_processor(
+        self, dialect: Dialect
+    ) -> Optional[_BindProcessorType[Union[decimal.Decimal, float]]]:
         return None
 
 
@@ -254,7 +308,7 @@ class INTEGER(_IntegerType, sqltypes.INTEGER):
 
     __visit_name__ = "INTEGER"
 
-    def __init__(self, display_width=None, **kw):
+    def __init__(self, display_width: Optional[int] = None, **kw: Any):
         """Construct an INTEGER.
 
         :param display_width: Optional, maximum display width for this number.
@@ -267,7 +321,7 @@ class INTEGER(_IntegerType, sqltypes.INTEGER):
           numeric.
 
         """
-        super(INTEGER, self).__init__(display_width=display_width, **kw)
+        super().__init__(display_width=display_width, **kw)
 
 
 class BIGINT(_IntegerType, sqltypes.BIGINT):
@@ -275,7 +329,7 @@ class BIGINT(_IntegerType, sqltypes.BIGINT):
 
     __visit_name__ = "BIGINT"
 
-    def __init__(self, display_width=None, **kw):
+    def __init__(self, display_width: Optional[int] = None, **kw: Any):
         """Construct a BIGINTEGER.
 
         :param display_width: Optional, maximum display width for this number.
@@ -288,7 +342,7 @@ class BIGINT(_IntegerType, sqltypes.BIGINT):
           numeric.
 
         """
-        super(BIGINT, self).__init__(display_width=display_width, **kw)
+        super().__init__(display_width=display_width, **kw)
 
 
 class MEDIUMINT(_IntegerType):
@@ -296,7 +350,7 @@ class MEDIUMINT(_IntegerType):
 
     __visit_name__ = "MEDIUMINT"
 
-    def __init__(self, display_width=None, **kw):
+    def __init__(self, display_width: Optional[int] = None, **kw: Any):
         """Construct a MEDIUMINTEGER
 
         :param display_width: Optional, maximum display width for this number.
@@ -309,7 +363,7 @@ class MEDIUMINT(_IntegerType):
           numeric.
 
         """
-        super(MEDIUMINT, self).__init__(display_width=display_width, **kw)
+        super().__init__(display_width=display_width, **kw)
 
 
 class TINYINT(_IntegerType):
@@ -317,7 +371,7 @@ class TINYINT(_IntegerType):
 
     __visit_name__ = "TINYINT"
 
-    def __init__(self, display_width=None, **kw):
+    def __init__(self, display_width: Optional[int] = None, **kw: Any):
         """Construct a TINYINT.
 
         :param display_width: Optional, maximum display width for this number.
@@ -330,7 +384,13 @@ class TINYINT(_IntegerType):
           numeric.
 
         """
-        super(TINYINT, self).__init__(display_width=display_width, **kw)
+        super().__init__(display_width=display_width, **kw)
+
+    def _compare_type_affinity(self, other: TypeEngine[Any]) -> bool:
+        return (
+            self._type_affinity is other._type_affinity
+            or other._type_affinity is sqltypes.Boolean
+        )
 
 
 class SMALLINT(_IntegerType, sqltypes.SMALLINT):
@@ -338,7 +398,7 @@ class SMALLINT(_IntegerType, sqltypes.SMALLINT):
 
     __visit_name__ = "SMALLINT"
 
-    def __init__(self, display_width=None, **kw):
+    def __init__(self, display_width: Optional[int] = None, **kw: Any):
         """Construct a SMALLINTEGER.
 
         :param display_width: Optional, maximum display width for this number.
@@ -351,10 +411,10 @@ class SMALLINT(_IntegerType, sqltypes.SMALLINT):
           numeric.
 
         """
-        super(SMALLINT, self).__init__(display_width=display_width, **kw)
+        super().__init__(display_width=display_width, **kw)
 
 
-class BIT(sqltypes.TypeEngine):
+class BIT(sqltypes.TypeEngine[Any]):
     """MySQL BIT type.
 
     This type is for MySQL 5.0.3 or greater for MyISAM, and 5.0.5 or greater
@@ -365,7 +425,7 @@ class BIT(sqltypes.TypeEngine):
 
     __visit_name__ = "BIT"
 
-    def __init__(self, length=None):
+    def __init__(self, length: Optional[int] = None):
         """Construct a BIT.
 
         :param length: Optional, number of bits.
@@ -373,20 +433,19 @@ class BIT(sqltypes.TypeEngine):
         """
         self.length = length
 
-    def result_processor(self, dialect, coltype):
-        """Convert a MySQL's 64 bit, variable length binary string to a long.
+    def result_processor(
+        self, dialect: MySQLDialect, coltype: object  # type: ignore[override]
+    ) -> Optional[_ResultProcessorType[Any]]:
+        """Convert a MySQL's 64 bit, variable length binary string to a
+        long."""
 
-        TODO: this is MySQL-db, pyodbc specific.  OurSQL and mysqlconnector
-        already do this, so this logic should be moved to those dialects.
+        if dialect.supports_native_bit:
+            return None
 
-        """
-
-        def process(value):
+        def process(value: Optional[Iterable[int]]) -> Optional[int]:
             if value is not None:
                 v = 0
                 for i in value:
-                    if not isinstance(i, int):
-                        i = ord(i)  # convert byte to int on Python 2
                     v = v << 8 | i
                 return v
             return value
@@ -399,7 +458,7 @@ class TIME(sqltypes.TIME):
 
     __visit_name__ = "TIME"
 
-    def __init__(self, timezone=False, fsp=None):
+    def __init__(self, timezone: bool = False, fsp: Optional[int] = None):
         """Construct a MySQL TIME type.
 
         :param timezone: not used by the MySQL dialect.
@@ -415,13 +474,15 @@ class TIME(sqltypes.TIME):
             MySQL Connector/Python.
 
         """
-        super(TIME, self).__init__(timezone=timezone)
+        super().__init__(timezone=timezone)
         self.fsp = fsp
 
-    def result_processor(self, dialect, coltype):
+    def result_processor(
+        self, dialect: Dialect, coltype: object
+    ) -> _ResultProcessorType[datetime.time]:
         time = datetime.time
 
-        def process(value):
+        def process(value: Any) -> Optional[datetime.time]:
             # convert from a timedelta value
             if value is not None:
                 microseconds = value.microseconds
@@ -444,7 +505,7 @@ class TIMESTAMP(sqltypes.TIMESTAMP):
 
     __visit_name__ = "TIMESTAMP"
 
-    def __init__(self, timezone=False, fsp=None):
+    def __init__(self, timezone: bool = False, fsp: Optional[int] = None):
         """Construct a MySQL TIMESTAMP type.
 
         :param timezone: not used by the MySQL dialect.
@@ -460,7 +521,7 @@ class TIMESTAMP(sqltypes.TIMESTAMP):
             MySQL Connector/Python.
 
         """
-        super(TIMESTAMP, self).__init__(timezone=timezone)
+        super().__init__(timezone=timezone)
         self.fsp = fsp
 
 
@@ -469,7 +530,7 @@ class DATETIME(sqltypes.DATETIME):
 
     __visit_name__ = "DATETIME"
 
-    def __init__(self, timezone=False, fsp=None):
+    def __init__(self, timezone: bool = False, fsp: Optional[int] = None):
         """Construct a MySQL DATETIME type.
 
         :param timezone: not used by the MySQL dialect.
@@ -485,16 +546,16 @@ class DATETIME(sqltypes.DATETIME):
             MySQL Connector/Python.
 
         """
-        super(DATETIME, self).__init__(timezone=timezone)
+        super().__init__(timezone=timezone)
         self.fsp = fsp
 
 
-class YEAR(sqltypes.TypeEngine):
+class YEAR(sqltypes.TypeEngine[Any]):
     """MySQL YEAR type, for single byte storage of years 1901-2155."""
 
     __visit_name__ = "YEAR"
 
-    def __init__(self, display_width=None):
+    def __init__(self, display_width: Optional[int] = None):
         self.display_width = display_width
 
 
@@ -503,7 +564,7 @@ class TEXT(_StringType, sqltypes.TEXT):
 
     __visit_name__ = "TEXT"
 
-    def __init__(self, length=None, **kw):
+    def __init__(self, length: Optional[int] = None, **kw: Any):
         """Construct a TEXT.
 
         :param length: Optional, if provided the server may optimize storage
@@ -531,7 +592,7 @@ class TEXT(_StringType, sqltypes.TEXT):
           only the collation of character data.
 
         """
-        super(TEXT, self).__init__(length=length, **kw)
+        super().__init__(length=length, **kw)
 
 
 class TINYTEXT(_StringType):
@@ -539,7 +600,7 @@ class TINYTEXT(_StringType):
 
     __visit_name__ = "TINYTEXT"
 
-    def __init__(self, **kwargs):
+    def __init__(self, **kwargs: Any):
         """Construct a TINYTEXT.
 
         :param charset: Optional, a column-level character set for this string
@@ -563,7 +624,7 @@ class TINYTEXT(_StringType):
           only the collation of character data.
 
         """
-        super(TINYTEXT, self).__init__(**kwargs)
+        super().__init__(**kwargs)
 
 
 class MEDIUMTEXT(_StringType):
@@ -572,7 +633,7 @@ class MEDIUMTEXT(_StringType):
 
     __visit_name__ = "MEDIUMTEXT"
 
-    def __init__(self, **kwargs):
+    def __init__(self, **kwargs: Any):
         """Construct a MEDIUMTEXT.
 
         :param charset: Optional, a column-level character set for this string
@@ -596,7 +657,7 @@ class MEDIUMTEXT(_StringType):
           only the collation of character data.
 
         """
-        super(MEDIUMTEXT, self).__init__(**kwargs)
+        super().__init__(**kwargs)
 
 
 class LONGTEXT(_StringType):
@@ -604,7 +665,7 @@ class LONGTEXT(_StringType):
 
     __visit_name__ = "LONGTEXT"
 
-    def __init__(self, **kwargs):
+    def __init__(self, **kwargs: Any):
         """Construct a LONGTEXT.
 
         :param charset: Optional, a column-level character set for this string
@@ -628,7 +689,7 @@ class LONGTEXT(_StringType):
           only the collation of character data.
 
         """
-        super(LONGTEXT, self).__init__(**kwargs)
+        super().__init__(**kwargs)
 
 
 class VARCHAR(_StringType, sqltypes.VARCHAR):
@@ -636,7 +697,7 @@ class VARCHAR(_StringType, sqltypes.VARCHAR):
 
     __visit_name__ = "VARCHAR"
 
-    def __init__(self, length=None, **kwargs):
+    def __init__(self, length: Optional[int] = None, **kwargs: Any) -> None:
         """Construct a VARCHAR.
 
         :param charset: Optional, a column-level character set for this string
@@ -660,7 +721,7 @@ class VARCHAR(_StringType, sqltypes.VARCHAR):
           only the collation of character data.
 
         """
-        super(VARCHAR, self).__init__(length=length, **kwargs)
+        super().__init__(length=length, **kwargs)
 
 
 class CHAR(_StringType, sqltypes.CHAR):
@@ -668,7 +729,7 @@ class CHAR(_StringType, sqltypes.CHAR):
 
     __visit_name__ = "CHAR"
 
-    def __init__(self, length=None, **kwargs):
+    def __init__(self, length: Optional[int] = None, **kwargs: Any):
         """Construct a CHAR.
 
         :param length: Maximum data length, in characters.
@@ -681,10 +742,10 @@ class CHAR(_StringType, sqltypes.CHAR):
           compatible with the national character set.
 
         """
-        super(CHAR, self).__init__(length=length, **kwargs)
+        super().__init__(length=length, **kwargs)
 
     @classmethod
-    def _adapt_string_for_cast(cls, type_):
+    def _adapt_string_for_cast(cls, type_: sqltypes.String) -> sqltypes.CHAR:
         # copy the given string type into a CHAR
         # for the purposes of rendering a CAST expression
         type_ = sqltypes.to_instance(type_)
@@ -713,7 +774,7 @@ class NVARCHAR(_StringType, sqltypes.NVARCHAR):
 
     __visit_name__ = "NVARCHAR"
 
-    def __init__(self, length=None, **kwargs):
+    def __init__(self, length: Optional[int] = None, **kwargs: Any):
         """Construct an NVARCHAR.
 
         :param length: Maximum data length, in characters.
@@ -727,7 +788,7 @@ class NVARCHAR(_StringType, sqltypes.NVARCHAR):
 
         """
         kwargs["national"] = True
-        super(NVARCHAR, self).__init__(length=length, **kwargs)
+        super().__init__(length=length, **kwargs)
 
 
 class NCHAR(_StringType, sqltypes.NCHAR):
@@ -739,7 +800,7 @@ class NCHAR(_StringType, sqltypes.NCHAR):
 
     __visit_name__ = "NCHAR"
 
-    def __init__(self, length=None, **kwargs):
+    def __init__(self, length: Optional[int] = None, **kwargs: Any):
         """Construct an NCHAR.
 
         :param length: Maximum data length, in characters.
@@ -753,7 +814,7 @@ class NCHAR(_StringType, sqltypes.NCHAR):
 
         """
         kwargs["national"] = True
-        super(NCHAR, self).__init__(length=length, **kwargs)
+        super().__init__(length=length, **kwargs)
 
 
 class TINYBLOB(sqltypes._Binary):
