@@ -1,5 +1,5 @@
 # engine/url.py
-# Copyright (C) 2005-2024 the SQLAlchemy authors and contributors
+# Copyright (C) 2005-2026 the SQLAlchemy authors and contributors
 # <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
@@ -14,31 +14,36 @@ argument; alternatively, the URL is a public-facing construct which can
 be used directly and is also accepted directly by ``create_engine()``.
 """
 
+from __future__ import annotations
+
+import collections.abc as collections_abc
 import re
+from typing import Any
+from typing import cast
+from typing import Dict
+from typing import Iterable
+from typing import List
+from typing import Mapping
+from typing import NamedTuple
+from typing import Optional
+from typing import overload
+from typing import Sequence
+from typing import Tuple
+from typing import Type
+from typing import Union
+from urllib.parse import parse_qsl
+from urllib.parse import quote
+from urllib.parse import quote_plus
+from urllib.parse import unquote
 
 from .interfaces import Dialect
 from .. import exc
 from .. import util
 from ..dialects import plugins
 from ..dialects import registry
-from ..util import collections_abc
-from ..util import compat
 
 
-class URL(
-    util.namedtuple(
-        "URL",
-        [
-            "drivername",
-            "username",
-            "password",
-            "host",
-            "port",
-            "database",
-            "query",
-        ],
-    )
-):
+class URL(NamedTuple):
     """
     Represent the components of a URL used to connect to a database.
 
@@ -69,6 +74,10 @@ class URL(
         :class:`_engine.URL` object with modifications.   See notes for this
         change at :ref:`change_5526`.
 
+    .. seealso::
+
+        :ref:`database_urls`
+
     :class:`_engine.URL` contains the following attributes:
 
     * :attr:`_engine.URL.drivername`: database backend and driver name, such as
@@ -85,31 +94,78 @@ class URL(
 
     """
 
-    def __new__(self, *arg, **kw):
-        if kw.pop("_new_ok", False):
-            return super(URL, self).__new__(self, *arg, **kw)
-        else:
-            util.warn_deprecated(
-                "Calling URL() directly is deprecated and will be disabled "
-                "in a future release.  The public constructor for URL is "
-                "now the URL.create() method.",
-                "1.4",
-            )
-            return URL.create(*arg, **kw)
+    drivername: str
+    """database backend and driver name, such as
+    ``postgresql+psycopg2``
+
+    """
+
+    username: Optional[str]
+    "username string"
+
+    password: Optional[str]
+    """password, which is normally a string but may also be any
+    object that has a ``__str__()`` method."""
+
+    host: Optional[str]
+    """hostname or IP number.  May also be a data source name for some
+    drivers."""
+
+    port: Optional[int]
+    """integer port number"""
+
+    database: Optional[str]
+    """database name"""
+
+    query: util.immutabledict[str, Union[Tuple[str, ...], str]]
+    """an immutable mapping representing the query string.  contains strings
+       for keys and either strings or tuples of strings for values, e.g.::
+
+            >>> from sqlalchemy.engine import make_url
+            >>> url = make_url(
+            ...     "postgresql+psycopg2://user:pass@host/dbname?alt_host=host1&alt_host=host2&ssl_cipher=%2Fpath%2Fto%2Fcrt"
+            ... )
+            >>> url.query
+            immutabledict({'alt_host': ('host1', 'host2'), 'ssl_cipher': '/path/to/crt'})
+
+         To create a mutable copy of this mapping, use the ``dict`` constructor::
+
+            mutable_query_opts = dict(url.query)
+
+       .. seealso::
+
+          :attr:`_engine.URL.normalized_query` - normalizes all values into sequences
+          for consistent processing
+
+          Methods for altering the contents of :attr:`_engine.URL.query`:
+
+          :meth:`_engine.URL.update_query_dict`
+
+          :meth:`_engine.URL.update_query_string`
+
+          :meth:`_engine.URL.update_query_pairs`
+
+          :meth:`_engine.URL.difference_update_query`
+
+    """  # noqa: E501
 
     @classmethod
     def create(
         cls,
-        drivername,
-        username=None,
-        password=None,
-        host=None,
-        port=None,
-        database=None,
-        query=util.EMPTY_DICT,
-    ):
+        drivername: str,
+        username: Optional[str] = None,
+        password: Optional[str] = None,
+        host: Optional[str] = None,
+        port: Optional[int] = None,
+        database: Optional[str] = None,
+        query: Mapping[str, Union[Sequence[str], str]] = util.EMPTY_DICT,
+    ) -> URL:
         """Create a new :class:`_engine.URL` object.
 
+        .. seealso::
+
+            :ref:`database_urls`
+
         :param drivername: the name of the database backend. This name will
           correspond to a module in sqlalchemy/databases or a third party
           plug-in.
@@ -117,6 +173,11 @@ class URL(
         :param password: database password.  Is typically a string, but may
           also be an object that can be stringified with ``str()``.
 
+          .. note:: The password string should **not** be URL encoded when
+             passed as an argument to :meth:`_engine.URL.create`; the string
+             should contain the password characters exactly as they would be
+             typed.
+
           .. note::  A password-producing object will be stringified only
              **once** per :class:`_engine.Engine` object.  For dynamic password
              generation per connect, see :ref:`engines_dynamic_tokens`.
@@ -152,11 +213,10 @@ class URL(
             cls._assert_port(port),
             cls._assert_none_str(database, "database"),
             cls._str_dict(query),
-            _new_ok=True,
         )
 
     @classmethod
-    def _assert_port(cls, port):
+    def _assert_port(cls, port: Optional[int]) -> Optional[int]:
         if port is None:
             return None
         try:
@@ -165,25 +225,47 @@ class URL(
             raise TypeError("Port argument must be an integer or None")
 
     @classmethod
-    def _assert_str(cls, v, paramname):
-        if not isinstance(v, compat.string_types):
+    def _assert_str(cls, v: str, paramname: str) -> str:
+        if not isinstance(v, str):
             raise TypeError("%s must be a string" % paramname)
         return v
 
     @classmethod
-    def _assert_none_str(cls, v, paramname):
+    def _assert_none_str(
+        cls, v: Optional[str], paramname: str
+    ) -> Optional[str]:
         if v is None:
             return v
 
         return cls._assert_str(v, paramname)
 
     @classmethod
-    def _str_dict(cls, dict_):
+    def _str_dict(
+        cls,
+        dict_: Optional[
+            Union[
+                Sequence[Tuple[str, Union[Sequence[str], str]]],
+                Mapping[str, Union[Sequence[str], str]],
+            ]
+        ],
+    ) -> util.immutabledict[str, Union[Tuple[str, ...], str]]:
         if dict_ is None:
             return util.EMPTY_DICT
 
-        def _assert_value(val):
-            if isinstance(val, compat.string_types):
+        @overload
+        def _assert_value(
+            val: str,
+        ) -> str: ...
+
+        @overload
+        def _assert_value(
+            val: Sequence[str],
+        ) -> Union[str, Tuple[str, ...]]: ...
+
+        def _assert_value(
+            val: Union[str, Sequence[str]],
+        ) -> Union[str, Tuple[str, ...]]:
+            if isinstance(val, str):
                 return val
             elif isinstance(val, collections_abc.Sequence):
                 return tuple(_assert_value(elem) for elem in val)
@@ -193,11 +275,12 @@ class URL(
                     "sequences of strings"
                 )
 
-        def _assert_str(v):
-            if not isinstance(v, compat.string_types):
+        def _assert_str(v: str) -> str:
+            if not isinstance(v, str):
                 raise TypeError("Query dictionary keys must be strings")
             return v
 
+        dict_items: Iterable[Tuple[str, Union[Sequence[str], str]]]
         if isinstance(dict_, collections_abc.Sequence):
             dict_items = dict_
         else:
@@ -214,14 +297,14 @@ class URL(
 
     def set(
         self,
-        drivername=None,
-        username=None,
-        password=None,
-        host=None,
-        port=None,
-        database=None,
-        query=None,
-    ):
+        drivername: Optional[str] = None,
+        username: Optional[str] = None,
+        password: Optional[str] = None,
+        host: Optional[str] = None,
+        port: Optional[int] = None,
+        database: Optional[str] = None,
+        query: Optional[Mapping[str, Union[Sequence[str], str]]] = None,
+    ) -> URL:
         """return a new :class:`_engine.URL` object with modifications.
 
         Values are used if they are non-None.  To set a value to ``None``
@@ -247,7 +330,7 @@ class URL(
 
         """
 
-        kw = {}
+        kw: Dict[str, Any] = {}
         if drivername is not None:
             kw["drivername"] = drivername
         if username is not None:
@@ -263,10 +346,10 @@ class URL(
         if query is not None:
             kw["query"] = query
 
-        return self._replace(**kw)
+        return self._assert_replace(**kw)
 
-    def _replace(self, **kw):
-        """Override ``namedtuple._replace()`` to provide argument checking."""
+    def _assert_replace(self, **kw: Any) -> URL:
+        """argument checks before calling _replace()"""
 
         if "drivername" in kw:
             self._assert_str(kw["drivername"], "drivername")
@@ -278,19 +361,23 @@ class URL(
         if "query" in kw:
             kw["query"] = self._str_dict(kw["query"])
 
-        return super(URL, self)._replace(**kw)
+        return self._replace(**kw)
 
-    def update_query_string(self, query_string, append=False):
+    def update_query_string(
+        self, query_string: str, append: bool = False
+    ) -> URL:
         """Return a new :class:`_engine.URL` object with the :attr:`_engine.URL.query`
         parameter dictionary updated by the given query string.
 
         E.g.::
 
             >>> from sqlalchemy.engine import make_url
-            >>> url = make_url("postgresql://user:pass@host/dbname")
-            >>> url = url.update_query_string("alt_host=host1&alt_host=host2&ssl_cipher=%2Fpath%2Fto%2Fcrt")
+            >>> url = make_url("postgresql+psycopg2://user:pass@host/dbname")
+            >>> url = url.update_query_string(
+            ...     "alt_host=host1&alt_host=host2&ssl_cipher=%2Fpath%2Fto%2Fcrt"
+            ... )
             >>> str(url)
-            'postgresql://user:pass@host/dbname?alt_host=host1&alt_host=host2&ssl_cipher=%2Fpath%2Fto%2Fcrt'
+            'postgresql+psycopg2://user:pass@host/dbname?alt_host=host1&alt_host=host2&ssl_cipher=%2Fpath%2Fto%2Fcrt'
 
         :param query_string: a URL escaped query string, not including the
          question mark.
@@ -309,11 +396,13 @@ class URL(
             :meth:`_engine.URL.update_query_dict`
 
         """  # noqa: E501
-        return self.update_query_pairs(
-            util.parse_qsl(query_string), append=append
-        )
+        return self.update_query_pairs(parse_qsl(query_string), append=append)
 
-    def update_query_pairs(self, key_value_pairs, append=False):
+    def update_query_pairs(
+        self,
+        key_value_pairs: Iterable[Tuple[str, Union[str, List[str]]]],
+        append: bool = False,
+    ) -> URL:
         """Return a new :class:`_engine.URL` object with the
         :attr:`_engine.URL.query`
         parameter dictionary updated by the given sequence of key/value pairs
@@ -321,10 +410,16 @@ class URL(
         E.g.::
 
             >>> from sqlalchemy.engine import make_url
-            >>> url = make_url("postgresql://user:pass@host/dbname")
-            >>> url = url.update_query_pairs([("alt_host", "host1"), ("alt_host", "host2"), ("ssl_cipher", "/path/to/crt")])
+            >>> url = make_url("postgresql+psycopg2://user:pass@host/dbname")
+            >>> url = url.update_query_pairs(
+            ...     [
+            ...         ("alt_host", "host1"),
+            ...         ("alt_host", "host2"),
+            ...         ("ssl_cipher", "/path/to/crt"),
+            ...     ]
+            ... )
             >>> str(url)
-            'postgresql://user:pass@host/dbname?alt_host=host1&alt_host=host2&ssl_cipher=%2Fpath%2Fto%2Fcrt'
+            'postgresql+psycopg2://user:pass@host/dbname?alt_host=host1&alt_host=host2&ssl_cipher=%2Fpath%2Fto%2Fcrt'
 
         :param key_value_pairs: A sequence of tuples containing two strings
          each.
@@ -347,23 +442,27 @@ class URL(
         """  # noqa: E501
 
         existing_query = self.query
-        new_keys = {}
+        new_keys: Dict[str, Union[str, List[str]]] = {}
 
         for key, value in key_value_pairs:
             if key in new_keys:
                 new_keys[key] = util.to_list(new_keys[key])
-                new_keys[key].append(value)
+                cast("List[str]", new_keys[key]).append(cast(str, value))
             else:
-                new_keys[key] = value
+                new_keys[key] = (
+                    list(value) if isinstance(value, (list, tuple)) else value
+                )
 
+        new_query: Mapping[str, Union[str, Sequence[str]]]
         if append:
             new_query = {}
 
             for k in new_keys:
                 if k in existing_query:
-                    new_query[k] = util.to_list(
-                        existing_query[k]
-                    ) + util.to_list(new_keys[k])
+                    new_query[k] = tuple(
+                        util.to_list(existing_query[k])
+                        + util.to_list(new_keys[k])
+                    )
                 else:
                     new_query[k] = new_keys[k]
 
@@ -374,10 +473,19 @@ class URL(
                 }
             )
         else:
-            new_query = self.query.union(new_keys)
+            new_query = self.query.union(
+                {
+                    k: tuple(v) if isinstance(v, list) else v
+                    for k, v in new_keys.items()
+                }
+            )
         return self.set(query=new_query)
 
-    def update_query_dict(self, query_parameters, append=False):
+    def update_query_dict(
+        self,
+        query_parameters: Mapping[str, Union[str, List[str]]],
+        append: bool = False,
+    ) -> URL:
         """Return a new :class:`_engine.URL` object with the
         :attr:`_engine.URL.query` parameter dictionary updated by the given
         dictionary.
@@ -390,10 +498,12 @@ class URL(
 
 
             >>> from sqlalchemy.engine import make_url
-            >>> url = make_url("postgresql://user:pass@host/dbname")
-            >>> url = url.update_query_dict({"alt_host": ["host1", "host2"], "ssl_cipher": "/path/to/crt"})
+            >>> url = make_url("postgresql+psycopg2://user:pass@host/dbname")
+            >>> url = url.update_query_dict(
+            ...     {"alt_host": ["host1", "host2"], "ssl_cipher": "/path/to/crt"}
+            ... )
             >>> str(url)
-            'postgresql://user:pass@host/dbname?alt_host=host1&alt_host=host2&ssl_cipher=%2Fpath%2Fto%2Fcrt'
+            'postgresql+psycopg2://user:pass@host/dbname?alt_host=host1&alt_host=host2&ssl_cipher=%2Fpath%2Fto%2Fcrt'
 
 
         :param query_parameters: A dictionary with string keys and values
@@ -422,21 +532,21 @@ class URL(
         """  # noqa: E501
         return self.update_query_pairs(query_parameters.items(), append=append)
 
-    def difference_update_query(self, names):
+    def difference_update_query(self, names: Iterable[str]) -> URL:
         """
         Remove the given names from the :attr:`_engine.URL.query` dictionary,
         returning the new :class:`_engine.URL`.
 
         E.g.::
 
-            url = url.difference_update_query(['foo', 'bar'])
+            url = url.difference_update_query(["foo", "bar"])
 
         Equivalent to using :meth:`_engine.URL.set` as follows::
 
             url = url.set(
                 query={
                     key: url.query[key]
-                    for key in set(url.query).difference(['foo', 'bar'])
+                    for key in set(url.query).difference(["foo", "bar"])
                 }
             )
 
@@ -468,11 +578,10 @@ class URL(
                     for key in set(self.query).difference(names)
                 }
             ),
-            _new_ok=True,
         )
 
-    @util.memoized_property
-    def normalized_query(self):
+    @property
+    def normalized_query(self) -> Mapping[str, Sequence[str]]:
         """Return the :attr:`_engine.URL.query` dictionary with values normalized
         into sequences.
 
@@ -486,7 +595,9 @@ class URL(
 
 
             >>> from sqlalchemy.engine import make_url
-            >>> url = make_url("postgresql://user:pass@host/dbname?alt_host=host1&alt_host=host2&ssl_cipher=%2Fpath%2Fto%2Fcrt")
+            >>> url = make_url(
+            ...     "postgresql+psycopg2://user:pass@host/dbname?alt_host=host1&alt_host=host2&ssl_cipher=%2Fpath%2Fto%2Fcrt"
+            ... )
             >>> url.query
             immutabledict({'alt_host': ('host1', 'host2'), 'ssl_cipher': '/path/to/crt'})
             >>> url.normalized_query
@@ -507,7 +618,7 @@ class URL(
         "be removed in a future release.  Please use the "
         ":meth:`_engine.URL.render_as_string` method.",
     )
-    def __to_string__(self, hide_password=True):
+    def __to_string__(self, hide_password: bool = True) -> str:
         """Render this :class:`_engine.URL` object as a string.
 
         :param hide_password: Defaults to True.   The password is not shown
@@ -516,7 +627,7 @@ class URL(
         """
         return self.render_as_string(hide_password=hide_password)
 
-    def render_as_string(self, hide_password=True):
+    def render_as_string(self, hide_password: bool = True) -> str:
         """Render this :class:`_engine.URL` object as a string.
 
         This method is used when the ``__str__()`` or ``__repr__()``
@@ -528,17 +639,17 @@ class URL(
         """
         s = self.drivername + "://"
         if self.username is not None:
-            s += _sqla_url_quote(self.username)
+            s += quote(self.username, safe=" +")
             if self.password is not None:
                 s += ":" + (
                     "***"
                     if hide_password
-                    else _sqla_url_quote(str(self.password))
+                    else quote(str(self.password), safe=" +")
                 )
             s += "@"
         if self.host is not None:
             if ":" in self.host:
-                s += "[%s]" % self.host
+                s += f"[{self.host}]"
             else:
                 s += self.host
         if self.port is not None:
@@ -549,19 +660,16 @@ class URL(
             keys = list(self.query)
             keys.sort()
             s += "?" + "&".join(
-                "%s=%s" % (util.quote_plus(k), util.quote_plus(element))
+                f"{quote_plus(k)}={quote_plus(element)}"
                 for k in keys
                 for element in util.to_list(self.query[k])
             )
         return s
 
-    def __str__(self):
-        return self.render_as_string(hide_password=False)
-
-    def __repr__(self):
+    def __repr__(self) -> str:
         return self.render_as_string()
 
-    def __copy__(self):
+    def __copy__(self) -> URL:
         return self.__class__.create(
             self.drivername,
             self.username,
@@ -574,13 +682,13 @@ class URL(
             self.query,
         )
 
-    def __deepcopy__(self, memo):
+    def __deepcopy__(self, memo: Any) -> URL:
         return self.__copy__()
 
-    def __hash__(self):
+    def __hash__(self) -> int:
         return hash(str(self))
 
-    def __eq__(self, other):
+    def __eq__(self, other: Any) -> bool:
         return (
             isinstance(other, URL)
             and self.drivername == other.drivername
@@ -592,10 +700,10 @@ class URL(
             and self.port == other.port
         )
 
-    def __ne__(self, other):
+    def __ne__(self, other: Any) -> bool:
         return not self == other
 
-    def get_backend_name(self):
+    def get_backend_name(self) -> str:
         """Return the backend name.
 
         This is the name that corresponds to the database backend in
@@ -608,7 +716,7 @@ class URL(
         else:
             return self.drivername.split("+")[0]
 
-    def get_driver_name(self):
+    def get_driver_name(self) -> str:
         """Return the backend name.
 
         This is the name that corresponds to the DBAPI driver in
@@ -626,7 +734,9 @@ class URL(
         else:
             return self.drivername.split("+")[1]
 
-    def _instantiate_plugins(self, kwargs):
+    def _instantiate_plugins(
+        self, kwargs: Mapping[str, Any]
+    ) -> Tuple[URL, List[Any], Dict[str, Any]]:
         plugin_names = util.to_list(self.query.get("plugin", ()))
         plugin_names += kwargs.get("plugins", [])
 
@@ -648,7 +758,7 @@ class URL(
 
         return u, loaded_plugins, kwargs
 
-    def _get_entrypoint(self):
+    def _get_entrypoint(self) -> Type[Dialect]:
         """Return the "entry point" dialect class.
 
         This is normally the dialect itself except in the case when the
@@ -670,18 +780,23 @@ class URL(
         ):
             return cls.dialect
         else:
-            return cls
+            return cast("Type[Dialect]", cls)
 
-    def get_dialect(self):
+    def get_dialect(self, _is_async: bool = False) -> Type[Dialect]:
         """Return the SQLAlchemy :class:`_engine.Dialect` class corresponding
         to this URL's driver name.
 
         """
         entrypoint = self._get_entrypoint()
-        dialect_cls = entrypoint.get_dialect_cls(self)
+        if _is_async:
+            dialect_cls = entrypoint.get_async_dialect_cls(self)
+        else:
+            dialect_cls = entrypoint.get_dialect_cls(self)
         return dialect_cls
 
-    def translate_connect_args(self, names=None, **kw):
+    def translate_connect_args(
+        self, names: Optional[List[str]] = None, **kw: Any
+    ) -> Dict[str, Any]:
         r"""Translate url attributes into a dictionary of connection arguments.
 
         Returns attributes of this url (`host`, `database`, `username`,
@@ -721,9 +836,8 @@ class URL(
         return translated
 
 
-def make_url(name_or_url):
-    """Given a string or unicode instance, produce a new URL instance.
-
+def make_url(name_or_url: Union[str, URL]) -> URL:
+    """Given a string, produce a new URL instance.
 
     The format of the URL generally follows `RFC-1738
     <https://www.ietf.org/rfc/rfc1738.txt>`_, with some exceptions, including
@@ -732,15 +846,25 @@ def make_url(name_or_url):
 
     If a :class:`.URL` object is passed, it is returned as is.
 
+    .. seealso::
+
+        :ref:`database_urls`
+
     """
 
-    if isinstance(name_or_url, util.string_types):
+    if isinstance(name_or_url, str):
         return _parse_url(name_or_url)
+    elif not isinstance(name_or_url, URL) and not hasattr(
+        name_or_url, "_sqla_is_testing_if_this_is_a_mock_object"
+    ):
+        raise exc.ArgumentError(
+            f"Expected string or URL object, got {name_or_url!r}"
+        )
     else:
         return name_or_url
 
 
-def _parse_url(name):
+def _parse_url(name: str) -> URL:
     pattern = re.compile(
         r"""
             (?P<name>[\w\+]+)://
@@ -764,15 +888,14 @@ def _parse_url(name):
     m = pattern.match(name)
     if m is not None:
         components = m.groupdict()
+        query: Optional[Dict[str, Union[str, List[str]]]]
         if components["query"] is not None:
             query = {}
 
-            for key, value in util.parse_qsl(components["query"]):
-                if util.py2k:
-                    key = key.encode("ascii")
+            for key, value in parse_qsl(components["query"]):
                 if key in query:
                     query[key] = util.to_list(query[key])
-                    query[key].append(value)
+                    cast("List[str]", query[key]).append(value)
                 else:
                     query[key] = value
         else:
@@ -780,10 +903,10 @@ def _parse_url(name):
         components["query"] = query
 
         if components["username"] is not None:
-            components["username"] = _sqla_url_unquote(components["username"])
+            components["username"] = unquote(components["username"])
 
         if components["password"] is not None:
-            components["password"] = _sqla_url_unquote(components["password"])
+            components["password"] = unquote(components["password"])
 
         ipv4host = components.pop("ipv4host")
         ipv6host = components.pop("ipv6host")
@@ -793,27 +916,9 @@ def _parse_url(name):
         if components["port"]:
             components["port"] = int(components["port"])
 
-        return URL.create(name, **components)
+        return URL.create(name, **components)  # type: ignore
 
     else:
         raise exc.ArgumentError(
-            "Could not parse SQLAlchemy URL from string '%s'" % name
+            "Could not parse SQLAlchemy URL from given URL string"
         )
-
-
-def _sqla_url_quote(text):
-    return re.sub(r"[:@/]", lambda m: "%%%X" % ord(m.group(0)), text)
-
-
-def _sqla_url_unquote(text):
-    return util.unquote(text)
-
-
-def _parse_keyvalue_args(name):
-    m = re.match(r"(\w+)://(.*)", name)
-    if m is not None:
-        (name, args) = m.group(1, 2)
-        opts = dict(util.parse_qsl(args))
-        return URL(name, *opts)
-    else:
-        return None
