@@ -1,5 +1,5 @@
 # sql/functions.py
-# Copyright (C) 2005-2025 the SQLAlchemy authors and contributors
+# Copyright (C) 2005-2026 the SQLAlchemy authors and contributors
 # <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
@@ -241,7 +241,8 @@ class FunctionElement(Executable, ColumnElement[_T], FromClause, Generative):
         r"""Return a :class:`_sql.TableValuedAlias` representation of this
         :class:`_functions.FunctionElement` with table-valued expressions added.
 
-        e.g.:
+        e.g. to use the SQLite form of ``generate_series()`` (including
+        hidden columns "start", "stop", "step"):
 
         .. sourcecode:: pycon+sql
 
@@ -258,17 +259,33 @@ class FunctionElement(Executable, ColumnElement[_T], FromClause, Generative):
             FROM generate_series(:generate_series_1, :generate_series_2) AS anon_1
             WHERE anon_1.value > :value_1{stop}
 
+        Backends like PostgreSQL need the accessed columns to be explicitly
+        named in "AS" clause.  To achieve this, use
+        :meth:`_sql.TableValuedAlias.render_derived`; be sure to consult the
+        :ref:`PostgreSQL-specific documentation for table valued functions
+        <postgresql_table_valued>` for additional examples:
+
+        .. sourcecode:: pycon+sql
+
+            >>> fn = func.generate_series(1, 5).table_valued("value").render_derived()
+
+            >>> print(select(fn))
+            {printsql}SELECT anon_1.value FROM
+            generate_series(:generate_series_1, :generate_series_2) AS anon_1(value){stop}
+
         A WITH ORDINALITY expression may be generated by passing the keyword
-        argument "with_ordinality":
+        argument :paramref:`.FunctionElement.table_valued.with_ordinality`,
+        illustrated below using PostgreSQL's syntax:
 
         .. sourcecode:: pycon+sql
 
             >>> fn = func.generate_series(4, 1, -1).table_valued(
             ...     "gen", with_ordinality="ordinality"
             ... )
-            >>> print(select(fn))
+            >>> print(select(fn.render_derived()))
             {printsql}SELECT anon_1.gen, anon_1.ordinality
-            FROM generate_series(:generate_series_1, :generate_series_2, :generate_series_3) WITH ORDINALITY AS anon_1
+            FROM generate_series(:generate_series_1, :generate_series_2, :generate_series_3)
+            WITH ORDINALITY AS anon_1(gen, ordinality)
 
         :param \*expr: A series of string column names that will be added to the
          ``.c`` collection of the resulting :class:`_sql.TableValuedAlias`
@@ -297,7 +314,7 @@ class FunctionElement(Executable, ColumnElement[_T], FromClause, Generative):
 
             :ref:`tutorial_functions_table_valued` - in the :ref:`unified_tutorial`
 
-            :ref:`postgresql_table_valued` - in the :ref:`postgresql_toplevel` documentation
+            :ref:`Table-Valued Functions on PostgreSQL <postgresql_table_valued>` - in the :ref:`postgresql_toplevel` documentation
 
             :meth:`_functions.FunctionElement.scalar_table_valued` - variant of
             :meth:`_functions.FunctionElement.table_valued` which delivers the
@@ -745,7 +762,9 @@ class FunctionElement(Executable, ColumnElement[_T], FromClause, Generative):
         # expressions against getitem.  This may need to be made
         # more portable if in the future we support other DBs
         # besides postgresql.
-        if against in (operators.getitem, operators.json_getitem_op):
+        if against is operators.getitem and isinstance(
+            self.type, sqltypes.ARRAY
+        ):
             return Grouping(self)
         else:
             return super().self_group(against=against)
@@ -1047,7 +1066,7 @@ class _FunctionGenerator:
         @overload
         def coalesce(
             self,
-            col: _ColumnExpressionArgument[_T],
+            col: _ColumnExpressionArgument[Optional[_T]],
             *args: _ColumnExpressionOrLiteralArgument[Any],
             **kwargs: Any,
         ) -> coalesce[_T]: ...
@@ -1055,14 +1074,14 @@ class _FunctionGenerator:
         @overload
         def coalesce(
             self,
-            col: _T,
+            col: Optional[_T],
             *args: _ColumnExpressionOrLiteralArgument[Any],
             **kwargs: Any,
         ) -> coalesce[_T]: ...
 
         def coalesce(
             self,
-            col: _ColumnExpressionOrLiteralArgument[_T],
+            col: _ColumnExpressionOrLiteralArgument[Optional[_T]],
             *args: _ColumnExpressionOrLiteralArgument[Any],
             **kwargs: Any,
         ) -> coalesce[_T]: ...
@@ -1661,7 +1680,42 @@ class ReturnTypeFromArgs(GenericFunction[_T]):
         super().__init__(*fn_args, **kwargs)
 
 
-class coalesce(ReturnTypeFromArgs[_T]):
+class ReturnTypeFromOptionalArgs(ReturnTypeFromArgs[_T]):
+    inherit_cache = True
+
+    @overload
+    def __init__(
+        self,
+        col: ColumnElement[_T],
+        *args: _ColumnExpressionOrLiteralArgument[Any],
+        **kwargs: Any,
+    ) -> None: ...
+
+    @overload
+    def __init__(
+        self,
+        col: _ColumnExpressionArgument[Optional[_T]],
+        *args: _ColumnExpressionOrLiteralArgument[Any],
+        **kwargs: Any,
+    ) -> None: ...
+
+    @overload
+    def __init__(
+        self,
+        col: Optional[_T],
+        *args: _ColumnExpressionOrLiteralArgument[Any],
+        **kwargs: Any,
+    ) -> None: ...
+
+    def __init__(
+        self,
+        *args: _ColumnExpressionOrLiteralArgument[Optional[_T]],
+        **kwargs: Any,
+    ) -> None:
+        super().__init__(*args, **kwargs)  # type: ignore
+
+
+class coalesce(ReturnTypeFromOptionalArgs[_T]):
     _has_args = True
     inherit_cache = True
 
