@@ -1,5 +1,5 @@
 # testing/provision.py
-# Copyright (C) 2005-2025 the SQLAlchemy authors and contributors
+# Copyright (C) 2005-2026 the SQLAlchemy authors and contributors
 # <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
@@ -9,6 +9,7 @@
 from __future__ import annotations
 
 import collections
+import contextlib
 import logging
 
 from . import config
@@ -16,9 +17,13 @@ from . import engines
 from . import util
 from .. import exc
 from .. import inspect
+from ..engine import Connection
+from ..engine import Engine
 from ..engine import url as sa_url
+from ..schema import sort_tables_and_constraints
 from ..sql import ddl
 from ..sql import schema
+from ..util import decorator
 
 
 log = logging.getLogger(__name__)
@@ -49,11 +54,16 @@ class register:
 
         return decorate
 
+    def call_original(self, cfg, *arg, **kw):
+        return self.fns["*"](cfg, *arg, **kw)
+
     def __call__(self, cfg, *arg, **kw):
         if isinstance(cfg, str):
             url = sa_url.make_url(cfg)
         elif isinstance(cfg, sa_url.URL):
             url = cfg
+        elif isinstance(cfg, (Engine, Connection)):
+            url = cfg.engine.url
         else:
             url = cfg.db.url
         backend = url.get_backend_name()
@@ -83,7 +93,9 @@ def setup_config(db_url, options, file_config, follower_ident):
     update_db_opts(db_url, db_opts, options)
     db_opts["scope"] = "global"
     eng = engines.testing_engine(db_url, db_opts)
+
     post_configure_engine(db_url, eng, follower_ident)
+
     eng.connect().close()
 
     cfg = config.Config.register(eng, db_opts, options, file_config)
@@ -200,6 +212,23 @@ def _generate_driver_urls(url, extra_drivers):
             yield new_url
 
 
+@register.init
+def is_preferred_driver(cfg, engine):
+    """Return True if the engine's URL is on the "default" driver, or
+    more generally the "preferred" driver to use for tests.
+
+    Backends can override this to make a different driver the "prefeferred"
+    driver that's not the default.
+
+    """
+    return (
+        engine.url._get_entrypoint()
+        is engine.url.set(
+            drivername=engine.url.get_backend_name()
+        )._get_entrypoint()
+    )
+
+
 @register.init
 def generate_driver_url(url, driver, query_str):
     backend = url.get_backend_name()
@@ -366,12 +395,25 @@ def update_db_opts(db_url, db_opts, options):
 
 @register.init
 def post_configure_engine(url, engine, follower_ident):
-    """Perform extra steps after configuring an engine for testing.
+    """Perform extra steps after configuring the main engine for testing.
 
     (For the internal dialects, currently only used by sqlite, oracle, mssql)
     """
 
 
+@register.init
+def post_configure_testing_engine(url, engine, options, scope):
+    """perform extra steps after configuring any engine within the
+    testing_engine() function.
+
+    this includes the main engine as well as most ad-hoc testing engines.
+
+    steps here should not get in the way of test cases that are looking
+    for events, etc.
+
+    """
+
+
 @register.init
 def follower_url_from_main(url, ident):
     """Create a connection URL for a dynamically-created test database.
@@ -500,3 +542,56 @@ def normalize_sequence(cfg, sequence):
     The default implementation does nothing
     """
     return sequence
+
+
+@register.init
+def allow_stale_update_impl(cfg):
+    return contextlib.nullcontext()
+
+
+@decorator
+def allow_stale_updates(fn, *arg, **kw):
+    """decorator around a test function that indicates the test will
+    be UPDATING rows that have been read and are now stale.
+
+    This normally doesn't require intervention except for mariadb 12
+    which now raises its own error for that, and we want to turn off
+    that setting just within the scope of the test that needs it
+    to be turned off (i.e. ORM stale version tests)
+
+    """
+    with allow_stale_update_impl(config._current):
+        return fn(*arg, **kw)
+
+
+@register.init
+def delete_from_all_tables(connection, cfg, metadata):
+    """an absolutely foolproof delete from all tables routine.
+
+    dialects should override this to add special instructions like
+    disable constraints etc.
+
+    """
+    savepoints = getattr(cfg.requirements, "savepoints", False)
+    if savepoints:
+        savepoints = savepoints.enabled
+
+    inspector = inspect(connection)
+
+    for table in reversed(
+        [
+            t
+            for (t, fks) in sort_tables_and_constraints(
+                metadata.tables.values()
+            )
+            if t is not None
+            # remember that inspector.get_table_names() is cached,
+            # so this emits SQL once per unique schema name
+            and t.name in inspector.get_table_names(schema=t.schema)
+        ]
+    ):
+        if savepoints:
+            with connection.begin_nested():
+                connection.execute(table.delete())
+        else:
+            connection.execute(table.delete())
